/*
  +|========================================================================================+
  +| Project     : GVG 100/110 PANEL I2C TO MIDI INTERFACE
  +| Authors     : Mike G7GTN, Trevor G8CJS
  +| Date        : May 2020
  +| Version     : 2.00a
  +| IDE Version : Ardunio 1.8.11
  +|========================================================================================+
  +|
  +| Purpose:-
  +|
  +|    To receive I2C Instructions from the G8CJS GVG Panel Control Dongle and send a
  +|    custom user defined MIDI note on each. The 74HCT4067 multiplexer in the original
  +|    GVG 100/110 panel can optionally be hijacked to send a MIDI Control Change from
  +|    each of the 12 panel potentiometers. The I2C Bus address has been defined as 7 but 
  +|    you can change this to an address of your choice.
  +|
  +| Libraries:-
  +|
  +|    https://github.com/arduino-libraries/MIDIUSB
  +|
  +|
  +| The above custom libray needs to be installed. From IDE Tools Manage
  +| Libraries a search for 'MIDIUSB' will allow you to install this.
  +|
  +|    https://github.com/ivanseidel/ArduinoThread
  +|
  +| The above custom libray needs to be installed. From IDE Tools Manage
  +| Libraries a search for 'Thread' will allow you to install this.
  +|
  +|    https://github.com/sumotoy/Multiplexer4067
  +|
  +| The 74HC4067 library needs to be downloaded as a ZIP file and installed
  +| in to your IDE library location manually. Once unziped remove the master
  +| reference at the end of the filename. A typical location (Windows) might 
  +| be C:\USERS\NAME\DOCUMENTS\ARDUNIO\LIBRARIES\
  +|
  +| Looking on your drive will allow you to determine the actual location 
  +| specific to your own setup.
  +|
  +| Once the three libraries are correctly installed the structure should look
  +| like the following:-
  +|
  +|      C:\USERS\NAME\DOCUMENTS\ARDUNIO\LIBRARIES\
  +|                              ArdunioThread           <SUB DIRECTORY>
  +|                              MIDIUSB                 <SUB DIRECTORY>
  +|                              Multiplexer4067         <SUB DIRECTORY>
  +|
  +| Uploading using the Ardunio IDE
  +| 
  +| Select the board type as Ardunio Micro and make sure the correct COM Port has
  +| been chosen. You can now press the Upload Icon or use the shortcut of CTRL+U
  +|
  +| Uploading using an external Windows Applicaton vs IDE code Compiling.
  +|
  +| You may find it more appropriate to make use of a pre-compiled HEX 
  +| file to get the code on to the Pro Micro. One such utility can be freely
  +| downloaded from 
  +|
  +| https://sourceforge.net/projects/freematics/files/FreematicsBuilder/ 
  +| 
  +| 1) Install the Windows Application
  +| 2) Select the MIDI.HEX file and make sure that the Ardunio is selected as
  +| "MICRO" do not change the speed setting from the 16MHZ default value, or if this
  +| is not automatically defaulted then use the drop down box to select 16MHZ
  +|
  +| References:-
  +|
  +| https://iankresyman.home.blog/2018/12/17/project-2-diy-midi-controller-with-arduino/
  +| https://www.vmix.com/software/download.aspx
  +|
  +| Ardunio Pro Micro I/O Pins:-
  +|
  +|       2 ---- SDA   ---- VIA 4K7 PULLUP RESISTOR ------- + 5V **
  +|       3 ---- SCL   ---- VIA 4K7 PULLUP RESISTOR ------- + 5V
  +|       15 --- GROUND TO SEND MIDI TEST NOTES
  +|       A3 --- 74HC4067  ---- PIN 1  ---- Z --- A3 ANALOG INPUT FROM 74HCT4067
  +|       4  --- 74HC4067  ---- PIN 10 ---- S0
  +|       5  --- 74HC4067  ---- PIN 11 ---- S1
  +|       6  --- 74HC4067  ---- PIN 13 ---- S2
  +|       7  --- 74HC4067 ----- PIN 14 ---- S3
  +|          --- 74HC4067 ----- PIN 15 ---- ENABLE ----- GND
  +|
  +| ** It is wise to make sure you have fitted the 4K7 pullup resistors for reliable
  +| I2C bus communications.
  +|
  +| https://www.element14.com/community/community/arduino/blog/2019/12/19/creating-a-multi-purpose-i2c-device-between-a-raspberry-pi-and-an-arduino
  +|========================================================================================
*/

#include "MIDIUSB.h"
#include <Wire.h>
#include "Multiplexer4067.h"	    // 74HCT4067 control
#include <Thread.h>
#include <ThreadController.h>
#include <EEPROM.h>

//#define showserial		  // Uncomment to see I2C Value

const byte i2cAddress = 7;        // Change address value to suit
const byte MAX_REC_BYTES = 3;     // Maximum received bytes allowed 
const byte VERSION = 0x01;
enum 
{  // Command        offset
    IDENTIFICATION = 1,           // Read bytes device ID and version 
    USER1          = 2,
    USER2          = 3,
    USER3          = 4,
    USER4          = 5,
    USER5          = 6,
    USER6          = 7,
};
char command;

// POTENTIONETERS
const byte NPots = 15; 		        // Number of analog inputs used
const byte muxPotPin[NPots] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14}; // From gvg1.jpg circuit diagram
int potCState[NPots] = {0}; 	    // Current status of the analog port
int potPState[NPots] = {0}; 	    // Previous status of the analog port
int potVar = 0; 		              // Varying delta between potCState[NPots] and potPState[NPots]
int lastCcValue[NPots] = {0};

// POTENTIOMETER READING
int TIMEOUT = 50; 		            // Amount of time potentiometer will be read after it exceeds the varThreshold value
byte varThreshold = 4; 		        // Threshold for the potentiometer signal variation
boolean potMoving = true; 	      // potMoving set to true
unsigned long pTime[NPots] = {0}; // Previously stored time
unsigned long timer[NPots] = {0}; // Stores the time that has elasped since the timer was reset

byte midiCh = 0; 		              // This is the default MIDI channel to be used
byte note = 36; 		              // Note to be used
byte cc = 1; 			                // Control Change Channel used
Multiplexer4067 mplex = Multiplexer4067(4, 5, 6, 7, A3);	// MILTIPLEXER
ThreadController cpu; 		        // Where other threads can be added
Thread threadReadPots; 		        // Thread to control the potentiometers

//+--------------------------------------------------------------------------------
//+ First parameter is the event type (0x09 = note on, 0x08 = note off).
//+ Second parameter is note-on/note-off, combined with the channel.
//+ Channel can be anything between 0-15. Typically reported to the user as 1-16.
//+ Third parameter is the note number (48 = middle C).
//+ Fourth parameter is the velocity (64 = normal, 127 = fastest).
//+--------------------------------------------------------------------------------
void noteOn(byte channel, byte pitch, byte velocity)
{
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
}

void noteOff(byte channel, byte pitch, byte velocity)
{
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
}

void controlChange(byte channel, byte control, byte value)
{
  midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
  MidiUSB.sendMIDI(event);
}

//+--------------------------------------------------------------------------------
//+ Hardware
//+--------------------------------------------------------------------------------
void setup()
{
  command = 0;
  pinMode(A3, INPUT_PULLUP);        // Analog from 74HC4067
  pinMode(15, INPUT_PULLUP);        // Ground for test notes
  if (digitalRead(15) == LOW) 	    // Send MIDI test notes, 10 second delay between each	
  {
      midiTestNotes();
  }
  Serial.begin(9600);
  Wire.begin (i2cAddress);
  Wire.onReceive (receiveEvent);    // Incoming data from GVG Dongle
  Wire.onRequest(requestEvent);  
  mplex.begin();                    // Initialize the 4067 Multiplexer

  threadReadPots.setInterval(10);   // threads
  threadReadPots.onRun(readPots);
  cpu.add(&threadReadPots);
}

//+--------------------------------------------------------------------------------
//+ Get Numerical I2C Command from GVG Hardware Dongle.
//+ Each button requires a unique value - see keymap.txt for details
//+--------------------------------------------------------------------------------
void receiveEvent (int howMany)
{
  while (Wire.available () > 0)
  {
    byte c;
    byte value;
    c = Wire.read ();
    switch (c)
    {
      case 0:		    // Keybus 10 buttons see keymap.txt file for numeric values
        note0();    // Send selected MIDI Note
        break;
      case 1:
        note1();
        break;
      case 2:
        note2();
        break;
      case 3:
        note3();
        break;
      case 4:
        note4();
        break;
      case 5:
        note5();
        break;
      case 6:
        note6();
        break;
      case 7:
        note7();
        break;
      case 8:
        note8();
        break;
      case 9:
        note9();
        break;
      case 10:		      // Program Bus 10 buttons
        note10();
        break;
      case 11:
        note11();
        break;
      case 12:
        note12();
        break;
      case 13:
        note13();
        break;
      case 14:
        note14();
        break;
      case 15:
        note15();
        break;
      case 16:
        note16();
        break;
      case 17:
        note17();
        break;
      case 18:
        note18();
        break;
      case 19:
        note19();
        break;
      case 20:		      // Preset Bus 10 buttons
        note20();
        break;
      case 21:
        note21();
        break;
      case 22:
        note22();
        break;
      case 23:
        note23();
        break;
      case 24:
        note24();
        break;
      case 25:
        note25();
        break;
      case 26:
        note26();
        break;
      case 27:
        note27();
        break;
      case 28:
        note28();
        break;
      case 29:
        note29();
        break;
      case 30:            // BKGD
        note30();
        break;
      case 31:            // KEY
        note31();
        break;
      case 32:            // MIX
        note32();
        break;
      case 33:            // WIPE
        note33();
        break;
      case 34:            // CUT
        note34();
        break;
      case 35:            // AUTO
        note35();
        break;
      case 36:            // FADE TO BLACK
        note36();
        break;
      case 37:            // DSK CUT
        note37();
        break;
      case 38:            // DSK MIX
        note38();
        break;
      case 39:            // ASPECT ON
        note39();
        break;
      case 40:            // KEY BUS
        note40();
        break;
      case 41:            // MATTE
        note41();
        break;
      case 42:            // PST
        note42();
        break;
      case 43:            // KEY BUS?
        note43();
        break;
      case 44 :            // EFF EXT
        note44();
        break;
      case 45:            // CHROMA
        note45();
        break;
      case 46:            // KEY MASK
        note46();
        break;
      case 47:            // KEY INV
        note47();
        break;
      case 48:            // EXT VIDEO
        note48();
        break;
      case 49:            // DSK MATTE
        note49();
        break;
      case 50:            // BRDER LINE
        note50();
        break;
      case 51:            // DROP SHAD
        note51();
        break;
      case 52:            // OUTLINE
        note52();
        break;
      case 53:            // DSK EXT
        note53();
        break;
      case 54:            // KEY BUS
        note54();
        break;
      case 55:            // DSK PVW
        note55();
        break;
      case 56:            // KEY MASK2
        note56();
        break;
      case 57:            // KEY INV2
        note57();
        break;
      case 58:            // POSITIONER
        note58();
        break;
      case 59:            // REVERSE
        note59();
        break;
      case 60:            // EMEM 1
        note60();
        break;
      case 61:            // EMEM 2
        note61();
        break;
      case 62:            // EMEM 3
        note62();
        break;
      case 63:            // EMEM 4
        note63();
        break;
      case 64:            // EMEM 5
        note64();
        break;
      case 65:            // EMEM 6
        note65();
        break;
      case 66:            // EMEM 7
        note66();
        break;
      case 67:            // EMEM 8
        note67();
        break;
      case 68:            // EMEM 9
        note68();
        break;
      case 69:            // EMEM 10
        note69();
        break;
      case 70:            // SELECT
        note70();
        break;
      case 71:            // EDITOR
        note71();
        break;
      case 72:            // FRAME 1
        note72();
        break;
      case 73:            // FRAME 2
        note73();
        break;
      case 74:            // FRAME 3
        note74();
        break;
      case 75:            // SELECT 2
        note75();
        break;
      case 76:            // T-BAR FADER
        //note76();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 5, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        }              
        break;   
      case 77:            // MASK
        //note77();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 6, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush(); 
        }
        break;
      case 78:            // ASPECT
        //note78();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 7, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        }
        break;     
      case 79:            // SOFT
        //note79();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 8, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush(); 
        }
        break;  
      case 80:            // BORDER
        //note80();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 9, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;  
      case 81:            // CHROMA KEY  
        //note81();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 10, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;  
      case 82:            // GAIN
        //note82();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 11, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;  
      case 83:            // CLIP  
        //note83();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 12, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;  
      case 84:            // HUE  
        //note84();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 13, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;
      case 85:            // CHROMA
        //note85();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 14, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        }  
        break;    
      case 86:            // LUM 
        //note86();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 15, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        } 
        break;
      case 87:            // CLIP/GAIN 
        //note87();
        while (Wire.available () > 0)   // Still have data?
        {
         byte b;
         b = Wire.read();
         #if defined showserial  
            Serial.println("RECEIVED NUMBER:");
            Serial.println(b);            // Send value to Serial  
         #endif
         controlChange(1, 16, (byte)b); // Set the value of MIDI CC to I2C Value
         MidiUSB.flush();
        }  
        break;     
      case 90:            // MIDI Note Off Commands
        noteOff1();
        break;
      case 91:
        noteOff2();
        break;
      case 92:
        noteOff3();
        break;
      case 93:
        noteOff4();
        break;
      case 94:
        noteOff5();
        break;
      case 95:
        noteOff6();
        break;
      case 96:
        noteOff7();
        break;
      case 97:
        noteOff8();
        break;
      case 98:
        noteOff9();
        break;
      case 99:
        noteOff10();
        break;
      case 100:
        noteOff11();
        break;
      case 101:
        noteOff12();
        break;
      case 102:
        noteOff13();
        break;
      case 103:
        noteOff14();
        break;
      case 104:
        noteOff15();
        break;
      case 105:
        noteOff16();
        break;
      case 106:
        noteOff17();
        break;
      case 107:
        noteOff18();
        break;
      case 108:
        noteOff19();
        break;
      case 109:
        noteOff20();
        break;
      case 110:
        noteOff21();
        break;
      case 111:
        noteOff22();
        break;
      case 112:
        noteOff23();
        break;
      case 113:
        noteOff24();
        break;
      case 114:
        noteOff25();
        break;
      case 115:
        noteOff26();
        break;
      case 116:
        noteOff27();
        break;
      case 117:
        noteOff28();
        break;
      case 118:
        noteOff29();
        break;
      case 119:
        noteOff30();
        break;
      case 120:
        noteOff31();
        break;
      case 121:
        noteOff32();
        break;
      case 122:
        noteOff33();
        break;
      case 123:
        noteOff34();
        break;
      case 124:
        noteOff35();
        break;
      case 125:
        noteOff36();
        break;
      case 126:
        noteOff37();
        break;
      case 127:
        noteOff38();
        break;
      case 128:
        noteOff39();
        break;
      case 129:
        noteOff40();
        break;
      case 130:
        noteOff41();
        break;
      case 131:
        noteOff42();
        break;
      case 132:
        noteOff43();
        break;
      case 133:
        noteOff44();
        break;
      case 134:
        noteOff45();
        break;
      case 135:
        noteOff46();
        break;
      case 136:
        noteOff47();
        break;
      case 137:
        noteOff48();
        break;
      case 138:
        noteOff49();
        break;
      case 139:
        noteOff50();
        break;
      case 140:
        noteOff51();
        break;
      case 141:
        noteOff52();
        break;
      case 142:
        noteOff53();
        break;
      case 143:
        noteOff54();
        break;
      case 144:
        noteOff55();
        break;
      case 145:
        noteOff56();
        break;
      case 146:
        noteOff57();
        break;
      case 147:
        noteOff58();
        break;
      case 148:
        noteOff59();
        break;
      case 149:
        noteOff60();
        break;
      // Send Test MIDI Notes on I2C write 300 
      // 10 Second delay between notes
      case 300:  
        midiTestNotes();
        break;
       // Detect Slave  
      case 310:
        value = 255;
        Wire.write(value);
        break; 
    }
  }
}

//+--------------------------------------------------------------------------------
//+ Requested I2C from Master
//+--------------------------------------------------------------------------------
void requestEvent()
{
 switch (command)
 {
  case IDENTIFICATION: 
    Wire.write (0xFF);          // Send ID 255 decimal
    break; 
  case USER1:
    Wire.write (1);             // Dummy value
    break; 
  case USER2:
    Wire.write (2);
    break;  
  case USER3:
    Wire.write (3);
    break;  
  case USER4:
    Wire.write (4);
    break; 
  case USER5:
    Wire.write (5);
    break;       
  case USER6:
    Wire.write (6);
    break;           
 }  
}

//+--------------------------------------------------------------------------------
//+ Send MIDI Note to VMIX
//+ Velocity can be between 1-127 - 64 is the midway point
//+--------------------------------------------------------------------------------
void note0()
{
  noteOn(2, 0, 64);   // Channel 2, note, normal velocity
  MidiUSB.flush();
}
void note1()
{
  noteOn(2, 1, 64);   // Channel 2, note, normal velocity
  MidiUSB.flush();
}
void note2()
{
  noteOn(2, 2, 64);
  MidiUSB.flush();
}
void note3()
{
  noteOn(2, 3, 64);
  MidiUSB.flush();
}
void note4()
{
  noteOn(2, 4, 64);
  MidiUSB.flush();
}
void note5()
{
  noteOn(2, 5, 64);
  MidiUSB.flush();
}
void note6()
{
  noteOn(2, 6, 64);
  MidiUSB.flush();
}
void note7()
{
  noteOn(2, 7, 64);
  MidiUSB.flush();
}
void note8()
{
  noteOn(2, 8, 64);
  MidiUSB.flush();
}
void note9()
{
  noteOn(2, 9, 64);
  MidiUSB.flush();
}
void note10()
{
  noteOn(2, 10, 64);
  MidiUSB.flush();
}
void note11()
{
  noteOn(2, 11, 64);
  MidiUSB.flush();
}
void note12()
{
  noteOn(2, 12, 64);
  MidiUSB.flush();
}
void note13()
{
  noteOn(2, 13, 64);
  MidiUSB.flush();
}
void note14()
{
  noteOn(2, 14, 64);
  MidiUSB.flush();
}
void note15()
{
  noteOn(2, 15, 64);
  MidiUSB.flush();
}
void note16()
{
  noteOn(2, 16, 64);
  MidiUSB.flush();
}
void note17()
{
  noteOn(2, 17, 64);
  MidiUSB.flush();
}
void note18()
{
  noteOn(2, 18, 64);
  MidiUSB.flush();
}
void note19()
{
  noteOn(2, 19, 64);
  MidiUSB.flush();
}
void note20()
{
  noteOn(2, 20, 64);
  MidiUSB.flush();
}
void note21()
{
  noteOn(2, 21, 64);
  MidiUSB.flush();
}
void note22()
{
  noteOn(2, 22, 64);
  MidiUSB.flush();
}
void note23()
{
  noteOn(2, 23, 64);
  MidiUSB.flush();
}
void note24()
{
  noteOn(2, 24, 64);
  MidiUSB.flush();
}
void note25()
{
  noteOn(2, 25, 64);
  MidiUSB.flush();
}
void note26()
{
  noteOn(2, 26, 64);
  MidiUSB.flush();
}
void note27()
{
  noteOn(2, 27, 64);
  MidiUSB.flush();
}
void note28()
{
  noteOn(2, 28, 64);
  MidiUSB.flush();
}
void note29()
{
  noteOn(2, 29, 64);
  MidiUSB.flush();
}
void note30()
{
  noteOn(2, 30, 64);
  MidiUSB.flush();
}
void note31()
{
  noteOn(2, 31, 64);
  MidiUSB.flush();
}
void note32()
{
  noteOn(2, 32, 64);
  MidiUSB.flush();
}
void note33()
{
  noteOn(2, 33, 64);
  MidiUSB.flush();
}
void note34()
{
  noteOn(2, 34, 64);
  MidiUSB.flush();
}
void note35()
{
  noteOn(2, 35, 64);
  MidiUSB.flush();
}
void note36()
{
  noteOn(2, 36, 64);
  MidiUSB.flush();
}
void note37()
{
  noteOn(2, 37, 64);
  MidiUSB.flush();
}
void note38()
{
  noteOn(2, 38, 64);
  MidiUSB.flush();
}
void note39()
{
  noteOn(2, 39, 64);
  MidiUSB.flush();
}
void note40()
{
  noteOn(2, 40, 64);
  MidiUSB.flush();
}
void note41()
{
  noteOn(2, 41, 64);
  MidiUSB.flush();
}
void note42()
{
  noteOn(2, 42, 64);
  MidiUSB.flush();
}
void note43()
{
  noteOn(2, 43, 64);
  MidiUSB.flush();
}
void note44()
{
  noteOn(2, 44, 64);
  MidiUSB.flush();
}
void note45()
{
  noteOn(2, 45, 64);
  MidiUSB.flush();
}
void note46()
{
  noteOn(2, 46, 64);
  MidiUSB.flush();
}
void note47()
{
  noteOn(2, 47, 64);
  MidiUSB.flush();
}
void note48()
{
  noteOn(2, 48, 64);
  MidiUSB.flush();
}
void note49()
{
  noteOn(2, 49, 64);
  MidiUSB.flush();
}
void note50()
{
  noteOn(2, 50, 64);
  MidiUSB.flush();
}
void note51()
{
  noteOn(2, 51, 64);
  MidiUSB.flush();
}
void note52()
{
  noteOn(2, 52, 64);
  MidiUSB.flush();
}
void note53()
{
  noteOn(2, 53, 64);
  MidiUSB.flush();
}
void note54()
{
  noteOn(2, 54, 64);
  MidiUSB.flush();
}
void note55()
{
  noteOn(2, 55, 64);
  MidiUSB.flush();
}
void note56()
{
  noteOn(2, 56, 64);
  MidiUSB.flush();
}
void note57()
{
  noteOn(2, 57, 64);
  MidiUSB.flush();
}
void note58()
{
  noteOn(2, 58, 64);
  MidiUSB.flush();
}
void note59()
{
  noteOn(2, 59, 64);
  MidiUSB.flush();
}
void note60()
{
  noteOn(2, 60, 64);
  MidiUSB.flush();
}
void note61()
{
  noteOn(2, 61, 64);
  MidiUSB.flush();
}
void note62()
{
  noteOn(2, 62, 64);
  MidiUSB.flush();
}
void note63()
{
  noteOn(2, 63, 64);
  MidiUSB.flush();
}
void note64()
{
  noteOn(2, 64, 64);
  MidiUSB.flush();
}
void note65()
{
  noteOn(2, 65, 64);
  MidiUSB.flush();
}
void note66()
{
  noteOn(2, 66, 64);
  MidiUSB.flush();
}
void note67()
{
  noteOn(2, 67, 64);
  MidiUSB.flush();
}
void note68()
{
  noteOn(2, 68, 64);
  MidiUSB.flush();
}
void note69()
{
  noteOn(2, 69, 64);
  MidiUSB.flush();
}
void note70()
{
  noteOn(2, 70, 64);
  MidiUSB.flush();
}
void note71()
{
  noteOn(2, 71, 64);
  MidiUSB.flush();
}
void note72()
{
  noteOn(2, 72, 64);
  MidiUSB.flush();
}
void note73()
{
  noteOn(2, 73, 64);
  MidiUSB.flush();
}
void note74()
{
  noteOn(2, 74, 64);
  MidiUSB.flush();
}
void note75()
{
  noteOn(2, 75, 64);
  MidiUSB.flush();
}
/*
void note76()                 // T-BAR
{
  while (Wire.available () > 0)	 // Still have data?
  {
   byte b;
   b = Wire.read();
   Serial.println("RECEIVED NUMBER:");
   Serial.println(b);            // Send value to Serial  
   controlChange(2, 5, (byte)b); // Set the value of MIDI controller to I2C Value
   MidiUSB.flush();		 // Send Value
  }
}
void note77()                 // MASK
{
  controlChange(2, 6, (byte)b));
  MidiUSB.flush();
}
void note78()                 // ASPECT
{
  controlChange(2, 7, (byte)b));
  MidiUSB.flush();
}
void note79()                 // SOFT
{
  controlChange(2, 8, (byte)b));
  MidiUSB.flush();
}
void note80()                 // BORDER
{
  controlChange(2, 9, (byte)b));
  MidiUSB.flush();
}
void note81()                 // CHROMA KEY
{
  controlChange(2, 10, (byte)b));
  MidiUSB.flush();
}
void note82()                 // GAIN       
{
 controlChange(2, 11, (byte)b));
  MidiUSB.flush();
}
void note83()                 // CLIP
{
  controlChange(2, 12, (byte)b));
  MidiUSB.flush();
}
void note84()                 // HUE
{
  controlChange(2, 13, (byte)b));
  MidiUSB.flush();
}
void note85()                 // CHROMA
{
  controlChange(2, 14, (byte)b));
  MidiUSB.flush();
}
void note86()                 // LUM 
{ 
  controlChange(2, 15, (byte)b));
  MidiUSB.flush();
}
void note87()                 // CLIP/GAIN
{
  controlChange(2, 16, (byte)b));
  MidiUSB.flush();
}
*/
void note88()
{
  noteOn(2, 88, 64);
  MidiUSB.flush();
}
void note89()
{
  noteOn(2, 89, 64);
  MidiUSB.flush();
}
void note90()
{
  noteOn(2, 90, 64);
  MidiUSB.flush();
}
void note91()
{
  noteOn(2, 91, 64);
  MidiUSB.flush();
}
void note92()
{
  noteOn(2, 92, 64);
  MidiUSB.flush();
}
void note93()
{
  noteOn(2, 93, 64);
  MidiUSB.flush();
}
void note94()
{
  noteOn(2, 94, 64);
  MidiUSB.flush();
}
void note95()
{
  noteOn(2, 95, 64);
  MidiUSB.flush();
}
void note96()
{
  noteOn(2, 96, 64);
  MidiUSB.flush();
}
void note97()
{
  noteOn(2, 97, 64);
  MidiUSB.flush();
}
void note98()
{
  noteOn(2, 98, 64);
  MidiUSB.flush();
}
void note99()
{
  noteOn(2, 99, 64);
  MidiUSB.flush();
}
void note100()
{
  noteOn(2, 100, 64);
  MidiUSB.flush();
}
void note101()
{
  noteOn(2, 101, 64);
  MidiUSB.flush();
}
void note102()
{
  noteOn(2, 102, 64);
  MidiUSB.flush();
}
void note103()
{
  noteOn(2, 103, 64);
  MidiUSB.flush();
}
void note104()
{
  noteOn(2, 104, 64);
  MidiUSB.flush();
}
void note105()
{
  noteOn(2, 105, 64);
  MidiUSB.flush();
}
void note106()
{
  noteOn(2, 106, 64);
  MidiUSB.flush();
}
void note107()
{
  noteOn(2, 107, 64);
  MidiUSB.flush();
}
void note108()
{
  noteOn(2, 108, 64);
  MidiUSB.flush();
}
void note109()
{
  noteOn(2, 109, 64);
  MidiUSB.flush();
}
void note110()
{
  noteOn(2, 110, 64);
  MidiUSB.flush();
}
void note111()
{
  noteOn(2, 111, 64);
  MidiUSB.flush();
}
void note112()
{
  noteOn(2, 112, 64);
  MidiUSB.flush();
}
void note113()
{
  noteOn(2, 113, 64);
  MidiUSB.flush();
}
void note114()
{
  noteOn(2, 114, 64);
  MidiUSB.flush();
}
void note115()
{
  noteOn(2, 115, 64);
  MidiUSB.flush();
}
void note116()
{
  noteOn(2, 116, 64);
  MidiUSB.flush();
}
void note117()
{
  noteOn(2, 117, 64);
  MidiUSB.flush();
}
void note118()
{
  noteOn(2, 118, 64);
  MidiUSB.flush();
}
void note119()
{
  noteOn(2, 119, 64);
  MidiUSB.flush();
}
void note120()
{
  noteOn(2, 120, 64);
  MidiUSB.flush();
}
void note121()
{
  noteOn(2, 121, 64);
  MidiUSB.flush();
}
void note122()
{
  noteOn(2, 122, 64);
  MidiUSB.flush();
}
void note123()
{
  noteOn(2, 123, 64);
  MidiUSB.flush();
}
void note124()
{
  noteOn(2, 124, 64);
  MidiUSB.flush();
}
void note125()
{
  noteOn(2, 125, 64);
  MidiUSB.flush();
}
void note126()
{
  noteOn(2, 126, 64);
  MidiUSB.flush();
}
void note127()
{
  noteOn(2, 127, 64);
  MidiUSB.flush();
}
// Midi Note Off Commands
// I2C range 80 - 99
void noteOff1()
{
  noteOff(2, 1, 64);   // Channel 2, note, normal velocity
  MidiUSB.flush();
}
void noteOff2()
{
  noteOff(2, 2, 64);
  MidiUSB.flush();
}
void noteOff3()
{
  noteOff(2, 3, 64);
  MidiUSB.flush();
}
void noteOff4()
{
  noteOff(2, 4, 64);
  MidiUSB.flush();
}
void noteOff5()
{
  noteOff(2, 5, 64);
  MidiUSB.flush();
}
void noteOff6()
{
  noteOff(2, 6, 64);
  MidiUSB.flush();
}
void noteOff7()
{
  noteOff(2, 7, 64);
  MidiUSB.flush();
}
void noteOff8()
{
  noteOff(2, 8, 64);
  MidiUSB.flush();
}
void noteOff9()
{
  noteOff(2, 9, 64);
  MidiUSB.flush();
}
void noteOff10()
{
  noteOff(2, 10, 64);
  MidiUSB.flush();
}
void noteOff11()
{
  noteOff(2, 11, 64);
  MidiUSB.flush();
}
void noteOff12()
{
  noteOff(2, 12, 64);
  MidiUSB.flush();
}
void noteOff13()
{
  noteOff(2, 13, 64);
  MidiUSB.flush();
}
void noteOff14()
{
  noteOff(2, 14, 64);
  MidiUSB.flush();
}
void noteOff15()
{
  noteOff(2, 15, 64);
  MidiUSB.flush();
}
void noteOff16()
{
  noteOff(2, 16, 64);
  MidiUSB.flush();
}
void noteOff17()
{
  noteOff(2, 17, 64);
  MidiUSB.flush();
}
void noteOff18()
{
  noteOff(2, 18, 64);
  MidiUSB.flush();
}
void noteOff19()
{
  noteOff(2, 19, 64);
  MidiUSB.flush();
}
void noteOff20()
{
  noteOff(2, 20, 64);
  MidiUSB.flush();
}
void noteOff21()
{
  noteOff(2, 21, 64);
  MidiUSB.flush();
}
void noteOff22()
{
  noteOff(2, 22, 64);
  MidiUSB.flush();
}
void noteOff23()
{
  noteOff(2, 23, 64);
  MidiUSB.flush();
}
void noteOff24()
{
  noteOff(2, 24, 64);
  MidiUSB.flush();
}
void noteOff25()
{
  noteOff(2, 25, 64);
  MidiUSB.flush();
}
void noteOff26()
{
  noteOff(2, 26, 64);
  MidiUSB.flush();
}
void noteOff27()
{
  noteOff(2, 27, 64);
  MidiUSB.flush();
}
void noteOff28()
{
  noteOff(2, 28, 64);
  MidiUSB.flush();
}
void noteOff29()
{
  noteOff(2, 29, 64);
  MidiUSB.flush();
}
void noteOff30()
{
  noteOff(2, 30, 64);
  MidiUSB.flush();
}
void noteOff31()
{
  noteOff(2, 31, 64);
  MidiUSB.flush();
}
void noteOff32()
{
  noteOff(2, 32, 64);
  MidiUSB.flush();
}
void noteOff33()
{
  noteOff(2, 33, 64);
  MidiUSB.flush();
}
void noteOff34()
{
  noteOff(2, 34, 64);
  MidiUSB.flush();
}
void noteOff35()
{
  noteOff(2, 35, 64);
  MidiUSB.flush();
}
void noteOff36()
{
  noteOff(2, 36, 64);
  MidiUSB.flush();
}
void noteOff37()
{
  noteOff(2, 37, 64);
  MidiUSB.flush();
}
void noteOff38()
{
  noteOff(2, 38, 64);
  MidiUSB.flush();
}
void noteOff39()
{
  noteOff(2, 39, 64);
  MidiUSB.flush();
}
void noteOff40()
{
  noteOff(2, 40, 64);
  MidiUSB.flush();
}
void noteOff41()
{
  noteOff(2, 41, 64);
  MidiUSB.flush();
}
void noteOff42()
{
  noteOff(2, 42, 64);
  MidiUSB.flush();
}
void noteOff43()
{
  noteOff(2, 43, 64);
  MidiUSB.flush();
}
void noteOff44()
{
  noteOff(2, 44, 64);
  MidiUSB.flush();
}
void noteOff45()
{
  noteOff(2, 45, 64);
  MidiUSB.flush();
}
void noteOff46()
{
  noteOff(2, 46, 64);
  MidiUSB.flush();
}
void noteOff47()
{
  noteOff(2, 47, 64);
  MidiUSB.flush();
}
void noteOff48()
{
  noteOff(2, 48, 64);
  MidiUSB.flush();
}
void noteOff49()
{
  noteOff(2, 49, 64);
  MidiUSB.flush();
}
void noteOff50()
{
  noteOff(2, 50, 64);
  MidiUSB.flush();
}
void noteOff51()
{
  noteOff(2, 51, 64);
  MidiUSB.flush();
}
void noteOff52()
{
  noteOff(2, 52, 64);
  MidiUSB.flush();
}
void noteOff53()
{
  noteOff(2, 53, 64);
  MidiUSB.flush();
}
void noteOff54()
{
  noteOff(2, 54, 64);
  MidiUSB.flush();
}
void noteOff55()
{
  noteOff(2, 55, 64);
  MidiUSB.flush();
}
void noteOff56()
{
  noteOff(2, 56, 64);
  MidiUSB.flush();
}
void noteOff57()
{
  noteOff(2, 57, 64);
  MidiUSB.flush();
}
void noteOff58()
{
  noteOff(2, 58, 64);
  MidiUSB.flush();
}
void noteOff59()
{
  noteOff(2, 59, 64);
  MidiUSB.flush();
}
void noteOff60()
{
  noteOff(2, 60, 64);
  MidiUSB.flush();
}

//+--------------------------------------------------------------------------------
//+ Send test MIDI Notes on 12C write 300 OR I/O Pin 15 Grounded
//+ Delay = 10 seconds between note sending
//+--------------------------------------------------------------------------------
void midiTestNotes()
{
  note1();
  delay(10000);
  note2();
  delay(10000);
  note3();
  delay(10000);
  note4();
  delay(10000);
  note5();
  delay(10000);
  note6();
  delay(10000);
  note7();
  delay(10000);
  note8();
  delay(10000);
  note9();
  delay(10000);
  note10();
  delay(10000);
  note11();
  delay(10000);
  note12();
  delay(10000);
  note13();
  delay(10000);
  note14();
  delay(10000);
  note15();
  delay(10000);
  note16();
  delay(10000);
  note17();
  delay(10000);
  note18();
  delay(10000);
  note19();
  delay(10000);
  note20();
  delay(10000);
}

//+--------------------------------------------------------------------------------
//+ Read Panel Potentiometers
//+--------------------------------------------------------------------------------
void readPots()
{

  for (int i = 0; i < NPots - 1; i++) { // MIDI channels being used minus dedicated MIDI channel exchange
    potCState[i] = mplex.readChannel(muxPotPin[i]);
  }

  for (int i = 0; i < NPots; i++) {

    potVar = abs(potCState[i] - potPState[i]); // Calculates variation of analog port

    if (potVar >= varThreshold) {  		// sets a threshold for the variance in the pot state, if it varies more than x it sends the cc message
      pTime[i] = millis(); 			// Stores the previous time
    }
    timer[i] = millis() - pTime[i]; 		// Reset time
    if (timer[i] < TIMEOUT) { 			// If the time is less than the max allowed time than the potentiometer is still moving
      potMoving = true;
    }
    else {
      potMoving = false;
    }

    if (potMoving == true) { 			// If the potentiometer is still moving, send the control change
      int ccValue = map(potCState[i], 22, 1022, 0, 127);
      if (lastCcValue[i] != ccValue) {
        controlChange(11, cc + i, ccValue); 	// send control change (channel, CC, value)
        MidiUSB.flush();
        potPState[i] = potCState[i]; 		// Sends the current reading of the potentiometer to compare with the next
        lastCcValue[i] = ccValue;
      }
    }
  }
}

// CALCULATES MIDI CHANNEL BASED ON POTENTIOMETER POSITION
int potMidiCh ()
{
  int potCh =  map(mplex.readChannel(muxPotPin[9]), 22, 1023, 0, 4);
  if (potCh == 4) {
    potCh = 3;
  }
  return potCh + midiCh;
}

//+--------------------------------------------------------------------------------
//+ Spin
//+--------------------------------------------------------------------------------
void loop()
{
  readPots();
  cpu.run();
}
